from datetime import datetime

from api.database import db_pool, PreparedStatement
from api.authentication import is_moderator


_SQL_GET_ANNOUNCEMENTS = PreparedStatement("""
SELECT * FROM "announcements"
WHERE NOT "deleted"
    AND (? OR (
            "visible"
        AND (("time_expire" IS NULL) OR "time_expire" > ?)
        AND (("time_publish" IS NULL) OR "time_publish" <= ?)
    ))
""")


def query_announcements():
    is_mod = is_moderator()
    current_time: datetime = datetime.now()
    announcements_db = db_pool.execute_read_statement_in_transaction(
        _SQL_GET_ANNOUNCEMENTS, is_mod, current_time, current_time)
    
    announcements_json = []
    for announcement_db in announcements_db:
        announcement_json = {
            "id": announcement_db["id"],
            "text": announcement_db["text"]
        }
        if is_mod:
            time_publish = announcement_db["time_publish"]
            time_expire = announcement_db["time_expire"]
            has_not_expired = time_expire is None or time_expire > current_time
            announcement_json["is_currently_visible"] = (announcement_db["visible"]
                                                         and (time_publish is None or time_publish <= current_time)
                                                         and has_not_expired)
            announcement_json["has_expired"] = not has_not_expired
        announcements_json.append(announcement_json)
        level_db: int = announcement_db["level"]
        if level_db == 0:
            announcement_json["type"] = "info"
            announcement_json["visibility"] = "only_main_page"
        elif level_db == 1:
            announcement_json["type"] = "info"
            announcement_json["visibility"] = "all_pages"
        elif level_db == 2:
            announcement_json["type"] = "warning"
            announcement_json["visibility"] = "all_pages"
        elif level_db == 3:
            announcement_json["type"] = "important"
            announcement_json["visibility"] = "all_pages_and_embed"
        else:
            raise Exception("Unknown announcement level: " + str(level_db))
    
    return announcements_json
