from datetime import datetime

from api.routes import *
from api.database import *

_SQL_GET_SORTER_LOG = PreparedStatement(f"""
SELECT "sortlog".*,
    "lectures"."title" as "lecture_title",
    "lectures"."time" as "lecture_time",
    "lectures"."course_id" as "course_id",
    "courses"."title" as "course_title",
    "videos"."id" as "video_row_id"
FROM "sortlog"
LEFT JOIN "lectures" ON ("sortlog"."lecture_id" = "lectures"."id")
LEFT JOIN "courses" ON ("lectures"."course_id" = "courses"."id")
LEFT JOIN "videos" ON ("sortlog"."video_id" = "videos"."id")
ORDER BY "sortlog"."when" DESC
LIMIT 50
""")
_SQL_GET_SORTER_ERROR_LOG = PreparedStatement(f"""
SELECT *
FROM "sorterrorlog"
ORDER BY "sorterrorlog"."when" DESC
""")
_SQL_DELETE_ERROR_LOG_ENTRY = PreparedStatement(f"""
UPDATE "sorterrorlog_data"
SET "deleted" = true
WHERE "id" = ?
""")


@api_route("/sorter/log", ["GET"], allow_while_readonly=True)
@api_moderator_route()
def api_route_sorter_log():
    log_db, error_log_db = db_pool.execute_read_statements_in_transaction(
        (_SQL_GET_SORTER_LOG, []),
        (_SQL_GET_SORTER_ERROR_LOG, [])
    )
    
    log_json = []
    for log_entry_db in log_db:
        time: datetime = log_entry_db["when"]
        log_entry_json = {
            "time": time.strftime(API_DATETIME_FORMAT),
            "file_path": log_entry_db["path"],
            "media_source_id": log_entry_db["video_id"]
        }
        log_json.append(log_entry_json)
        if log_entry_db["video_row_id"] is not None and log_entry_db["course_id"] is not None:
            log_entry_json["course_id"] = log_entry_db["course_id"]
            log_entry_json["course_full_name"] = log_entry_db["course_title"]
            log_entry_json["lecture_id"] = log_entry_db["lecture_id"]
            log_entry_json["lecture_title"] = log_entry_db["lecture_title"]
            log_entry_json["lecture_time"] = log_entry_db["lecture_time"].strftime(API_DATETIME_FORMAT)
    
    error_log_json = []
    for log_entry_db in error_log_db:
        time: datetime = log_entry_db["when"]
        log_entry_json = {
            "id": log_entry_db["id"],
            "time": time.strftime(API_DATETIME_FORMAT),
            "file_path": log_entry_db["path"]
        }
        error_log_json.append(log_entry_json)
        
        if log_entry_db["course_id"] >= 0:
            log_entry_json["matched_course_id"] = log_entry_db["course_id"]
        
        try:
            log_entry_json["matched_lectures_ids"] = list(map(
                lambda x: int(x),
                log_entry_db["matches"].split(",")
            ))
        except ValueError:  # pragma: no cover
            pass
    
    return {
        "log": log_json,
        "error_log": error_log_json
    }


@api_route("/sorter/error_log/<int:log_id>", ["DELETE"])
@api_moderator_route(require_csrf_token=True)
def api_route_delete_sorter_error_log(log_id: int):
    check_client_int(log_id, "URL.log_id")
    db_pool.execute_write_transaction(lambda trans: trans.execute_statement_and_commit(
        _SQL_DELETE_ERROR_LOG_ENTRY,
        log_id
    ))
    return {}
